// 片元着色器

precision mediump float;

// yuv分量数据
uniform sampler2D tex_y;
uniform sampler2D tex_u;
uniform sampler2D tex_v;

// 纹理坐标
varying vec2 v_texcoord;

// 特写区域(x, y, z, w), x、y、z、w取值都是0.0~1.0, 整个画面是(0.0, 0.0, 1.0, 1.0)
uniform vec4 closeupRect;
const float SIZE_LEVEL   = 0.3;
const float OFFSET_LEVEL = 0.7;
const float borderWidth  = 0.01;

uniform highp float distortionFactor;


// 对比度函数,cont对比度[-1.0, 1.0]
highp vec3 adjustContrast(highp vec3 color,  highp float cont) {
    cont += 1.0;
    return ((color - 0.5) * cont + 0.5);
}

vec4 yuvToRgb(vec2 uv) {

    // get source_uv
    highp vec2 source_uv = uv;//fragCoord.xy / iResolution.xy;

    // assume your distortion center coordinate is (0.5,0.5),you can use your distortion center instead.
    highp vec2 distortion_center = vec2(0.5, 0.5);

    // Define algorithm dependent variables
    highp float distortion_x;
    highp float distortion_y;
    highp float rr;
    highp float r2;
    highp float theta;

    // define distortion coefficient K1 and K2 ,In most cases we can only adjust K1. then K2 parameters can be adjusted more perfect Effect
    // K1 < 0 is pincushion distortion
    // K1 >=0 is barrel distortion
    highp float distortion_k1 = distortionFactor;
    highp float distortion_k2 = distortionFactor == 0.0 ? 0.0 : 0.5;
    highp vec2 dest_uv;

    //--------------------------Algorithm Start----------------------------------------
    rr = sqrt(pow(source_uv.x - distortion_center.x, 2.0) + pow(source_uv.y - distortion_center.y, 2.0));
    r2 = rr * (1.0 + distortion_k1*pow(rr, 2.0) + distortion_k2*pow(rr, 4.0));
    theta = atan(source_uv.x - distortion_center.x, source_uv.y - distortion_center.y);
    distortion_x = sin(theta) * r2 * 1.0;//1.0 is  scale factor
    distortion_y = cos(theta) * r2 * 1.0;//1.0 is  scale factor
    dest_uv.x = distortion_x + 0.5;
    dest_uv.y = distortion_y + 0.5;
    //--------------------------Algorithm End------------------------------------------

    mediump vec3 yuv;
    highp vec3 rgb;
    yuv.x = texture2D(tex_y, dest_uv).r;
    yuv.y = texture2D(tex_u, dest_uv).r - 0.5;
    yuv.z = texture2D(tex_v, dest_uv).r - 0.5;
    rgb = mat3(1, 1, 1,
            0, -0.3441, 1.772,
            1.402, -0.7141, 0) * yuv;

    // GWPlayer可能解码ColorRange原因渲染整体偏白（相对于IoTVideo，安卓iOS都是），临时增加一点对比度，待分析优化
    rgb = adjustContrast(rgb, 0.2);

    return vec4(rgb, 1);
}

void closeupView() {
    if (v_texcoord.x > OFFSET_LEVEL && v_texcoord.x < 1.0
        && v_texcoord.y > OFFSET_LEVEL && v_texcoord.y < 1.0)
    {
        // 将原图下采样到指定区域中
        vec2 newTexCoord = v_texcoord;
        newTexCoord -= OFFSET_LEVEL;
        newTexCoord = newTexCoord / (1.0 - OFFSET_LEVEL);

        // 特写框
        if (newTexCoord.x >= closeupRect.x && newTexCoord.x <= closeupRect.x + borderWidth &&
        newTexCoord.y >= closeupRect.y && newTexCoord.y <= closeupRect.y + closeupRect.w) {
            gl_FragColor =  vec4(1.0, 0, 0, 1.0);
        } else if (newTexCoord.x >= closeupRect.x + closeupRect.z && newTexCoord.x <= closeupRect.x + closeupRect.z + borderWidth &&
        newTexCoord.y >= closeupRect.y && newTexCoord.y <= closeupRect.y + closeupRect.w) {
            gl_FragColor =  vec4(0, 1.0, 0, 1.0);
        } else if (newTexCoord.y >= closeupRect.y && newTexCoord.y <= closeupRect.y + borderWidth &&
        newTexCoord.x >= closeupRect.x && newTexCoord.x <= closeupRect.x + closeupRect.z) {
            gl_FragColor =  vec4(0, 1.0, 1.0, 1.0);
        } else if (newTexCoord.y >= closeupRect.y + closeupRect.w  && newTexCoord.y <= closeupRect.y + closeupRect.w + borderWidth &&
        newTexCoord.x >= closeupRect.x && newTexCoord.x <= closeupRect.x + closeupRect.z + borderWidth) {
            gl_FragColor =  vec4(1.0, 1.0, 0, 1.0);
        } else {
            // 全貌画面
            gl_FragColor = yuvToRgb(newTexCoord);
        }
    } else {
        // 特写画面

        // 放大倍数以宽高更大者为准
        float SCALE_LEVEL = max(closeupRect.z, closeupRect.w);
        vec2 newTexCoord = v_texcoord;

        // 纠正closeupRect里xy+wh超出边界的情况
        float x = min(closeupRect.x, 1.0 - SCALE_LEVEL);
        float y = min(closeupRect.y, 1.0 - SCALE_LEVEL);
        newTexCoord.xy = vec2(x, y) + (SCALE_LEVEL * newTexCoord.xy);

        //原纹理坐标缩放之后再进行采样
        gl_FragColor = yuvToRgb(newTexCoord);
    }
}

void main() {
    if (closeupRect != vec4(0, 0, 1.0, 1.0)) {
        closeupView();
    } else {
        gl_FragColor = yuvToRgb(v_texcoord);
    }
}

